@@ -1,21 +1,19 @@
 package TOML::Tiny::Parser;
 # ABSTRACT: parser used by TOML::Tiny
-$TOML::Tiny::Parser::VERSION = '0.15';
+$TOML::Tiny::Parser::VERSION = '0.16';
 use utf8;
 use strict;
 use warnings;
 no warnings qw(experimental);
 use v5.18;
 
-use Carp;
-use Data::Dumper;
+use Carp qw(confess);
+use Data::Dumper qw(Dumper);
 use Encode qw(decode FB_CROAK);
-use TOML::Tiny::Util qw(is_strict_array);
-use TOML::Tiny::Grammar;
-
-require Math::BigFloat;
-require Math::BigInt;
-require TOML::Tiny::Tokenizer;
+use Math::BigFloat ();
+use Math::BigInt ();
+use TOML::Tiny::Grammar qw($TimeOffset);
+use TOML::Tiny::Tokenizer ();
 
 our $TRUE  = 1;
 our $FALSE = 0;
@@ -146,50 +144,49 @@ sub declare_key {
   my ($self, $token) = @_;
   my $key = $self->current_key || return;
 
-  for ($token->{type}) {
-    when ('inline_array') {
-      $self->parse_error($token, "duplicate key: $key")
-        if exists $self->{array_tables}{$key};
+  if ($token->{type} eq 'inline_array') {
+    $self->parse_error($token, "duplicate key: $key")
+      if exists $self->{array_tables}{$key};
 
-      $self->{arrays}{$key} = 1;
+    $self->{arrays}{$key} = 1;
+    return;
+  }
+
+  if ($token->{type} eq 'array_table') {
+    if (exists $self->{arrays}{$key}) {
+      $self->parse_error($token, "duplicate key: $key");
     }
 
-    when ('array_table') {
-      if (exists $self->{arrays}{$key}) {
+    $self->{array_tables}{$key} = 1;
+    return;
+  }
+
+  if ($token->{type} eq 'table') {
+    $self->parse_error($token, "duplicate key: $key")
+      if exists $self->{arrays}{$key}
+      || exists $self->{array_tables}{$key};
+
+    if (exists $self->{tables}{$key}) {
+      # Tables cannot be redefined, *except* when doing so within a goddamn
+      # table array. Gawd I hate TOML.
+      my $in_a_stupid_table_array = 0;
+      my $node = $self->{root};
+
+      for my $key ($self->get_keys) {
+        if (exists $node->{$key} && ref($node->{$key}) eq 'ARRAY') {
+          $in_a_stupid_table_array = 1;
+          last;
+        } else {
+          $node = $node->{$key};
+        }
+      }
+
+      unless ($in_a_stupid_table_array) {
         $self->parse_error($token, "duplicate key: $key");
       }
-
-      $self->{array_tables}{$key} = 1;
-    }
-
-    when ('table') {
-      $self->parse_error($token, "duplicate key: $key")
-        if exists $self->{arrays}{$key}
-        || exists $self->{array_tables}{$key};
-
-      if (exists $self->{tables}{$key}) {
-        # Tables cannot be redefined, *except* when doing so within a goddamn
-        # table array. Gawd I hate TOML.
-        my $in_a_stupid_table_array = 0;
-        my $node = $self->{root};
-
-        for my $key ($self->get_keys) {
-          if (exists $node->{$key} && ref($node->{$key}) eq 'ARRAY') {
-            $in_a_stupid_table_array = 1;
-            last;
-          } else {
-            $node = $node->{$key};
-          }
-        }
-
-        unless ($in_a_stupid_table_array) {
-          $self->parse_error($token, "duplicate key: $key");
-        }
-      }
-      else {
-        $self->{tables}{$key} = 1;
-      }
+      return;
     }
+    $self->{tables}{$key} = 1;
   }
 }
 
@@ -198,16 +195,20 @@ sub scan_to_key {
   my $keys = shift // [ $self->get_keys ];
   my $node = $self->{root};
 
+  KEY:
   for my $key (@$keys) {
     if (exists $node->{$key}) {
-      for (ref $node->{$key}) {
-        $node = $node->{$key}     when 'HASH';
-        $node = $node->{$key}[-1] when 'ARRAY';
-        default{
-          my $full_key = join '.', @$keys;
-          die "$full_key is already defined\n";
-        }
+      my $ref = ref $node->{$key};
+      if ( $ref eq 'HASH' ) {
+        $node = $node->{$key};
+        next KEY;
       }
+      if ( $ref eq 'ARRAY' ) {
+        $node = $node->{$key}[-1];
+        next KEY;
+      }
+      my $full_key = join '.', @$keys;
+      die "$full_key is already defined\n";
     }
     else {
       $node = $node->{$key} = {};
@@ -228,38 +229,36 @@ sub parse_table {
   $self->declare_key($token);
 
   TOKEN: while (my $token = $self->next_token) {
-    for ($token->{type}) {
-      next TOKEN when 'EOL';
+    my $type = $token->{type};
+    next TOKEN if $type eq 'EOL';
 
-      when ('key') {
-        $self->expect_type($self->next_token, 'assign');
-        $self->push_keys($token);
-        $self->set_key($self->next_token);
-        $self->pop_keys;
+    if ( $type eq 'key') {
+      $self->expect_type($self->next_token, 'assign');
+      $self->push_keys($token);
+      $self->set_key($self->next_token);
+      $self->pop_keys;
 
-        if (my $eol = $self->next_token) {
-          $self->expect_type($eol, 'EOL');
-        } else {
-          return;
-        }
-      }
-
-      when ('array_table') {
-        $self->pop_keys;
-        @_ = ($self, $token);
-        goto \&parse_array_table;
-      }
-
-      when ('table') {
-        $self->pop_keys;
-        @_ = ($self, $token);
-        goto \&parse_table;
-      }
-
-      default{
-        $self->parse_error($token, "expected key-value pair, table, or array of tables but got $_");
+      if (my $eol = $self->next_token) {
+        $self->expect_type($eol, 'EOL');
+      } else {
+        return;
       }
+      next TOKEN;
     }
+
+    if ($type eq 'array_table') {
+      $self->pop_keys;
+      @_ = ($self, $token);
+      goto \&parse_array_table;
+    }
+
+    if ( $type eq 'table') {
+      $self->pop_keys;
+      @_ = ($self, $token);
+      goto \&parse_table;
+    }
+
+    $self->parse_error($token, "expected key-value pair, table, or array of tables but got $type");
   }
 }
 
@@ -277,33 +276,32 @@ sub parse_array_table {
   $node->{$key} //= [];
   push @{ $node->{$key} }, {};
 
-  TOKEN: while (my $token = $self->next_token) {
-    for ($token->{type}) {
-      next TOKEN when 'EOL';
+  TOKEN:
+  while (my $token = $self->next_token) {
+    my $type = $token->{type};
+    next TOKEN if $type eq 'EOL';
 
-      when ('key') {
-        $self->expect_type($self->next_token, 'assign');
-        $self->push_keys($token);
-        $self->set_key($self->next_token);
-        $self->pop_keys;
-      }
-
-      when ('array_table') {
-        $self->pop_keys;
-        @_ = ($self, $token);
-        goto \&parse_array_table;
-      }
-
-      when ('table') {
-        $self->pop_keys;
-        @_ = ($self, $token);
-        goto \&parse_table;
-      }
-
-      default{
-        $self->parse_error($token, "expected key-value pair, table, or array of tables but got $_");
-      }
+    if ($type eq 'key') {
+      $self->expect_type($self->next_token, 'assign');
+      $self->push_keys($token);
+      $self->set_key($self->next_token);
+      $self->pop_keys;
+      next TOKEN;
     }
+
+    if ($type eq 'array_table') {
+      $self->pop_keys;
+      @_ = ($self, $token);
+      goto \&parse_array_table;
+    }
+
+    if ($type eq 'table') {
+      $self->pop_keys;
+      @_ = ($self, $token);
+      goto \&parse_table;
+    }
+
+    $self->parse_error($token, "expected key-value pair, table, or array of tables but got $type");
   }
 }
 
@@ -318,19 +316,16 @@ sub parse_value {
   my $self  = shift;
   my $token = shift;
 
-  for ($token->{type}) {
-    return $token->{value} when 'string';
-    return $self->inflate_float($token) when 'float';
-    return $self->inflate_integer($token) when 'integer';
-    return $self->{inflate_boolean}->($token->{value}) when 'bool';
-    return $self->parse_datetime($token) when 'datetime';
-    return $self->parse_inline_table($token) when 'inline_table';
-    return $self->parse_array($token) when 'inline_array';
+  my $type = $token->{type};
+  return $token->{value} if $type eq 'string';
+  return $self->inflate_float($token) if $type eq'float';
+  return $self->inflate_integer($token) if $type eq 'integer';
+  return $self->{inflate_boolean}->($token->{value}) if $type eq 'bool';
+  return $self->parse_datetime($token) if $type eq 'datetime';
+  return $self->parse_inline_table($token) if $type eq 'inline_table';
+  return $self->parse_array($token) if $type eq 'inline_array';
 
-    default{
-      $self->parse_error($token, "value expected (bool, number, string, datetime, inline array, inline table), but found $_");
-    }
-  }
+  $self->parse_error($token, "value expected (bool, number, string, datetime, inline array, inline table), but found $type");
 }
 
 #-------------------------------------------------------------------------------
@@ -367,20 +362,16 @@ sub parse_array {
     my $token = $self->next_token;
     $self->expect_type($token, $expect);
 
-    for ($token->{type}) {
-      when ('comma') {
-        $expect = 'EOL|inline_array_close|string|float|integer|bool|datetime|inline_table|inline_array';
-        next TOKEN;
-      }
-
-      next TOKEN when 'EOL';
-      last TOKEN when 'inline_array_close';
-
-      default{
-        push @array, $self->parse_value($token);
-        $expect = 'comma|EOL|inline_array_close';
-      }
+    if ( $token->{type} eq 'comma') {
+      $expect = 'EOL|inline_array_close|string|float|integer|bool|datetime|inline_table|inline_array';
+      next TOKEN;
     }
+
+    next TOKEN if $token->{type} eq 'EOL';
+    last TOKEN if $token->{type} eq 'inline_array_close';
+
+    push @array, $self->parse_value($token);
+    $expect = 'comma|EOL|inline_array_close';
   }
 
   return \@array;
@@ -397,43 +388,40 @@ sub parse_inline_table {
     my $token = $self->next_token;
     $self->expect_type($token, $expect);
 
-    for ($token->{type}) {
-      when ('comma') {
-        $expect = $self->{strict}
-          ? 'EOL|key'
-          : 'EOL|key|inline_table_close';
+    my $type = $token->{type};
+    if ($type eq 'comma') {
+      $expect = $self->{strict}
+        ? 'EOL|key'
+        : 'EOL|key|inline_table_close';
 
-        next TOKEN;
-      }
-
-      when ('key') {
-        $self->expect_type($self->next_token, 'assign');
-
-        my $node = $table;
-        my @keys = @{ $token->{value} };
-        my $key  = pop @keys;
-
-        for (@keys) {
-          $node->{$_} ||= {};
-          $node = $node->{$_};
-        }
-
-        if (exists $node->{$key}) {
-          $self->parse_error($token, 'duplicate key: ' .  join('.', map{ qq{"$_"} } @{ $token->{value} }));
-        } else {
-          $node->{ $key } = $self->parse_value($self->next_token);
-        }
-
-        $expect = 'comma|inline_table_close';
-        next TOKEN;
-      }
-
-      last TOKEN when 'inline_table_close';
-
-      default{
-        $self->parse_error($token, "inline table expected key-value pair, but found $_");
-      }
+      next TOKEN;
     }
+
+    if ($type eq 'key') {
+      $self->expect_type($self->next_token, 'assign');
+
+      my $node = $table;
+      my @keys = @{ $token->{value} };
+      my $key  = pop @keys;
+
+      for (@keys) {
+        $node->{$_} ||= {};
+        $node = $node->{$_};
+      }
+
+      if (exists $node->{$key}) {
+        $self->parse_error($token, 'duplicate key: ' .  join('.', map{ qq{"$_"} } @{ $token->{value} }));
+      } else {
+        $node->{ $key } = $self->parse_value($self->next_token);
+      }
+
+      $expect = 'comma|inline_table_close';
+      next TOKEN;
+    }
+
+    last TOKEN if $type eq 'inline_table_close';
+
+    $self->parse_error($token, "inline table expected key-value pair, but found $type");
   }
 
   return $table;
@@ -535,7 +523,7 @@ TOML::Tiny::Parser - parser used by TOML::Tiny
 
 =head1 VERSION
 
-version 0.15
+version 0.16
 
 =head1 AUTHOR
 
@@ -543,7 +531,7 @@ Jeff Ober <sysread@fastmail.fm>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2021 by Jeff Ober.
+This software is copyright (c) 2023 by Jeff Ober.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
