@@ -1,6 +1,6 @@
 package TOML::Tiny::Util;
 # ABSTRACT: utility functions used by TOML::Tiny
-$TOML::Tiny::Util::VERSION = '0.14';
+$TOML::Tiny::Util::VERSION = '0.16';
 use strict;
 use warnings;
 no warnings 'experimental';
@@ -14,6 +14,16 @@ our @EXPORT_OK = qw(
   is_strict_array
 );
 
+my @_type_map = (
+  [ qr{Float},      'float' ],
+  [ qr{Int},        'integer' ],
+  [ qr{Boolean},    'bool' ],
+  [ qr{^$Boolean},  'bool' ],
+  [ qr{^$Float},    'float' ],
+  [ qr{^$Integer},  'integer' ],
+  [ qr{^$DateTime}, 'float' ],
+);
+
 sub is_strict_array {
   my $arr = shift;
 
@@ -21,31 +31,25 @@ sub is_strict_array {
     my $value = $_;
     my $type;
 
-    for (ref $value) {
-      $type = 'array'   when 'ARRAY';
-      $type = 'table'   when 'HASH';
-
-      # Do a little heuristic guess-work
-      $type = 'float'   when /Float/;
-      $type = 'integer' when /Int/;
-      $type = 'bool'    when /Boolean/;
-
-      when ('') {
-        for ($value) {
-          $type = 'bool'     when /^$Boolean/;
-          $type = 'float'    when /^$Float/;
-          $type = 'integer'  when /^$Integer/;
-          $type = 'datetime' when /^$DateTime/;
-          default{ $type = 'string' };
-        }
-      }
-
-      default{
-        $type = $_;
+    my $ref = ref($value);
+    if ($ref eq 'ARRAY') {
+      $type = 'array';
+    }
+    elsif ($ref eq 'HASH') {
+      $type = 'table';
+    }
+    # Do a little heuristic guess-work
+    else {
+      for my $pair (@_type_map) {
+        if ( $ref =~ m{$pair->[0]} ) {
+          $type = $pair->[1];
+         }
+         last;
       }
     }
+    $type //= 'string';
 
-    $type;
+    return $type;
   } @$arr;
 
   my $t = shift @types;
@@ -72,7 +76,7 @@ TOML::Tiny::Util - utility functions used by TOML::Tiny
 
 =head1 VERSION
 
-version 0.14
+version 0.16
 
 =head1 AUTHOR
 
@@ -80,7 +84,7 @@ Jeff Ober <sysread@fastmail.fm>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2021 by Jeff Ober.
+This software is copyright (c) 2023 by Jeff Ober.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
